/****************************************************************************************/
/*  LcdDriver.c                                  																				*/			
/*  Driver para el manejo del LCD                       																*/			
/*                                                                     									*/			
/*  Lcd_Init(): Inicializa el LCD																												*/			
/*                                                                     									*/			
/*  Lcd_PutC(c): Escribe un caracter o string en el LCD.       													*/			
/*               Caracteres con significado especial:																		*/			
/*                      \f  Borra el LCD                              									*/			
/*                      \n  mueve el cursor al inicio de la segunda linea               */			
/*                      \b  Mueve el cursor una posicion para atras                     */			
/*                                                                     									*/			
/*  Lcd_GotoXY(x,y): Setea la posicion del cursor																				*/			
/*                                                                     									*/			
/*  Lcd_PutC(x,y): Retorna el caracter en la posicion actual del cursor									*/
/*																																											*/
/*	Lcd_OutStr: Funcion para escribir un string en el LCD																*/			
/*                                                                     									*/			
/****************************************************************************************/

/****************************************************************************************/
/*        Autor: Ing. Ivn Csar Sierra                                                 */
/*        Fecha de inicio: 25/01/2013                																		*/
/*        Fecha de finalizacion: 25/01/2013        																				*/
/*				Compilador: CCS																																*/
/*        Version: 1.0                                        													*/
/*        Actualizaciones:                                															*/
/****************************************************************************************/

// Pinout.
#define LCD_ENABLE_PIN  				PIN_A6 
#define LCD_RS_PIN      				PIN_A7 
#define LCD_RW_PIN      				PIN_A4 
#define LCD_DATA4       				PIN_A0 
#define LCD_DATA5       				PIN_A1 
#define LCD_DATA6       				PIN_A2 
#define LCD_DATA7       				PIN_A3

#define lcd_output_enable(x) 		output_bit (LCD_ENABLE_PIN, x)
#define lcd_enable_tris()  			output_drive (LCD_ENABLE_PIN)
#define lcd_output_rs(x) 				output_bit (LCD_RS_PIN, x)
#define lcd_rs_tris()  					output_drive (LCD_RS_PIN)
#define lcd_output_rw(x) 				output_bit (LCD_RW_PIN, x)
#define lcd_rw_tris()  					output_drive (LCD_RW_PIN)

#ifndef LCD_TYPE
   #define LCD_TYPE 						2     	// 0=5x7, 1=5x10, 2=2 lines
#endif

#ifndef LCD_LINE_TWO
   #define LCD_LINE_TWO 				0x40    // Direccion de la RAM para la segunda linea
#endif

#ifndef LCD_LINE_LENGTH
   #define LCD_LINE_LENGTH 			16			// numero de digitos por fila
#endif

BYTE Lcd_ReadNibble (void);

// funcion para leer un byte dese el puerto del LCD
BYTE Lcd_ReadByte (void) {
	
   BYTE low, high;

   output_float (LCD_DATA4);
   output_float (LCD_DATA5);
   output_float (LCD_DATA6);
   output_float (LCD_DATA7);
        
   lcd_output_rw (1);
   delay_cycles (1);
   lcd_output_enable (1);
   delay_cycles (1);
   high = Lcd_ReadNibble();
      
   lcd_output_enable (0);
   delay_cycles (1);
   lcd_output_enable (1);
   delay_us (1);
   low = Lcd_ReadNibble ();
      
   lcd_output_enable (0);

   output_drive (LCD_DATA4);
   output_drive (LCD_DATA5);
   output_drive (LCD_DATA6);
   output_drive (LCD_DATA7);

   return((high << 4) | low);
}

// funcion para leer un nibble desde puerto del LCD
BYTE Lcd_ReadNibble (void) {
	
   BYTE n = 0x00;

   // leer dato desde el puerto
   n |= input (LCD_DATA4);
   n |= input (LCD_DATA5) << 1;
   n |= input (LCD_DATA6) << 2;
   n |= input (LCD_DATA7) << 3;
   
   return (n);
}

// funcion para escribir un nibble en el puerto del LCD
void Lcd_SendNibble (BYTE n) {
	
   // escribir dato en el puerto
   output_bit (LCD_DATA4, bit_test(n, 0));
   output_bit (LCD_DATA5, bit_test(n, 1));
   output_bit (LCD_DATA6, bit_test(n, 2));
   output_bit (LCD_DATA7, bit_test(n, 3));
      
   delay_cycles (1);
   lcd_output_enable (1);
   delay_us (2);
   lcd_output_enable (0);
}

// funcion para enviar un dato/comando al LCD
void Lcd_SendByte (BYTE address, BYTE n) {

	lcd_enable_tris ();
  lcd_rs_tris ();
  lcd_rw_tris ();

  lcd_output_rs (0);
  while (bit_test (Lcd_ReadByte(), 7));
  lcd_output_rs (address);
  delay_cycles (1);
  lcd_output_rw (0);
  delay_cycles (1);
  lcd_output_enable (0);
  Lcd_SendNibble (n >> 4);
  Lcd_SendNibble (n & 0xf);
}

// funcion para inicializar el display
void Lcd_Init (void) {

  output_drive (LCD_DATA4);
  output_drive (LCD_DATA5);
  output_drive (LCD_DATA6);
  output_drive (LCD_DATA7);

  lcd_enable_tris ();
  lcd_rs_tris ();
  lcd_rw_tris ();

  lcd_output_rs (0);
  lcd_output_rw (0);
  lcd_output_enable (0);
   
  delay_ms (15);
	
	// inicializa el display en 4 bits
  Lcd_SendNibble (0x2);
  delay_us (100);
  Lcd_SendNibble (0x2);
  delay_us (100);
  Lcd_SendNibble (0x2);
  delay_us (100);
  Lcd_SendNibble (0x2);
  Lcd_SendNibble (0xC);
  delay_us (100);
  Lcd_SendNibble (0x2);
  Lcd_SendNibble (0xC);
  delay_us (5);
  
  // configura el display luego del arranque
  Lcd_SendByte (0, 0x0C);
  delay_ms (2);
  Lcd_SendByte (0, 0x01);
}

// funcion para mover el cursor a la coordenadas fijadas
void Lcd_GotoXY (BYTE x, BYTE y) {
	
   BYTE address;
   
	if(y != 1)
  	address = LCD_LINE_TWO;
  else
  	address = 0;
     
	address += x - 1;
  Lcd_SendByte (0,0x80 | address);
}

// funcion para escribir un char o string
void Lcd_PutC (char c) {
	
	switch (c) {
  	case '\a':
    	Lcd_GotoXY (1, 1);
    break;

    case '\f':
    	Lcd_SendByte (0, 1);
      delay_ms (2);
    break;

    case '\n':
     	Lcd_GotoXY (1, 2);
    break;    
     
    case '\b':
    	Lcd_SendByte (0, 0x10);
		break;

    default:
    	Lcd_SendByte (1, c);
    break;
  }
}

// funcion para leer un char desde el display
char Lcd_GetC (BYTE x, BYTE y) {
   char value;

   Lcd_GotoXY (x,y);
   while (bit_test (Lcd_ReadByte (), 7)); // espero hasta que busy flag sea 0
   lcd_output_rs (1);
   value = Lcd_ReadByte ();
   lcd_output_rs (0);
   
   return (value);
}

// funcion para escribir un string en el LCD
void Lcd_OutStr (BYTE x , BYTE y, char *ptr_str) {
	
	// posicion el cursor
	Lcd_GotoXY (x, y);
	
	// envio los caracteres
	do {
		Lcd_PutC (*ptr_str);
		ptr_str++;		
	} while(*ptr_str != 0x00);
	
	return;
}
